/*
	Word Twiddle
	JavaScript game engine
	Version 0.9
	Copyright 2011 by Andrew Krywaniuk
	
	License is hereby granted to freely distribute this game for personal and non-commercial use.
	For other uses, please contact the author.
*/


// Run-time stats.
var g_total_anagrams = 0;
var g_anagrams_found = 0;
var g_running_score = 0;


var g_session_stats = null;
var g_career_stats = null;


// init_stats_obj - initialize a stats object for either session/career stats.
function init_stats_obj()
{
	var obj = new Object();
	obj.rounds_played = 0;
	obj.perfect_rounds = 0;
	obj.high_score = 0;
	
	// Cummulative stats for MIN(rounds_played, 100)
	obj.cum_score = 0;
//	obj.cum_score_max = 0; // not supported yet
	obj.cum_words_found = 0;
	obj.cum_words_max = 0;
	obj.cum_time_played = 0;
	
	return obj;
}


// init_session_stats - initialize the long term stats objects.
function init_session_stats()
{
	g_session_stats = init_stats_obj();
//	g_career_stats = init_stats_obj();
	load_career_stats_cookie();
}


// Convert the stats object to a string.
function serialize_stats(obj)
{
	var ar = [ "A", obj.rounds_played, obj.perfect_rounds, obj.high_score, obj.cum_score, 0, obj.cum_words_found, obj.words_max, obj.cum_time_played ];
	
	var str = ar.join("+");
	return str;
}

// Load the stats object from a string.
function unserialize_stats(str)
{
	var obj = init_stats_obj();
	var ar = str.split("+");

	// Sanity check the array.
	if (ar[0] != "A" || ar.length < 9)
		return obj;
			
	obj.rounds_played = parseInt(ar[1]);
	obj.perfect_rounds = parseInt(ar[2]);
	obj.high_score = parseInt(ar[3]);
	obj.cum_score = parseInt(ar[4]);
	obj.cum_words_found = parseInt(ar[6]);
	obj.words_max = parseInt(ar[7]);
	obj.cum_time_played = parseInt(ar[8]);
	
	return obj;
}



// load_career_stats_cookie - called at startup to initialize the career stats.
function load_career_stats_cookie()
{
	var str = readCookie("wt_career_stats");
	
	if (str)
	{
		var obj = unserialize_stats(str);
		g_career_stats = obj;
	}
	else
	{
		g_career_stats = init_stats_obj();
	}
}

function save_career_stats_cookie()
{
	var str = serialize_stats(g_career_stats);
	createCookie("wt_career_stats", str ,356);
}


function reset_session_stats()
{
	g_session_stats = init_stats_obj();
}

function reset_career_stats()
{
	g_career_stats = init_stats_obj();
	save_career_stats_cookie();
}


// __aggregate_round_stats - update stats for either session/career stats.
function __aggregate_round_stats(obj)
{
	obj.rounds_played++;
	if (g_anagrams_found == g_total_anagrams) obj.perfect_rounds++;
	if (g_running_score > obj.high_score) obj.high_score = g_running_score;


	
	// Once we have reached 100 rounds, just use an approximate moving average.
	if (obj.rounds_played > 100)
	{
		obj.cum_score *= 0.99;
		obj.cum_words_found *= 0.99;
		obj.cum_words_max *= 0.99;
		obj.cum_time_played *= 0.99;
	}
	
	obj.cum_score += g_running_score;
	obj.cum_words_found += g_anagrams_found;
	obj.cum_words_max += g_total_anagrams;
	obj.cum_time_played += (g_round_time - g_time_remaining);
}

// aggregate_round_stats - update stats based on the current round
function aggregate_round_stats()
{
	__aggregate_round_stats(g_session_stats);
	__aggregate_round_stats(g_career_stats);
	
	save_career_stats_cookie();
}

